<?php

class CfRBAC
{
    /**
     * @var PDO
     */
    private $dbConnection;

    const RBAC_PERMISSIONS_TABLE = 'rbac_permissions';
    const RBAC_ROLE_PERMISSION_TABLE = 'rbac_role_permission';

    /**
     * CfRole constructor.
     */
    public function __construct()
    {
        $this->dbConnection = CfPdo::getInstance()->getConnection();
    }

    public function getPermissionsByRoles(array $roles)
    {
        // use default permissions if user has no roles
        if(sizeof($roles) == 1 && $roles[0] == '') {
            $query = sprintf('SELECT * FROM %s WHERE allowed_by_default = true  ORDER BY "application", "group", "name"', self::RBAC_PERMISSIONS_TABLE);
        } else {
            $query = sprintf('SELECT * FROM %s WHERE alias IN (SELECT permission_alias FROM %s WHERE role_id IN (\'%s\')) ORDER BY "application", "group", "name"',
                    self::RBAC_PERMISSIONS_TABLE,
                    self::RBAC_ROLE_PERMISSION_TABLE,
                    implode("','", $roles));
        }

       return $this->dbConnection->query($query)->fetchAll(PDO::FETCH_ASSOC);
    }

    public function assignPermission(string $role, string $alias)
    {
        $this->dbConnection->query(
            sprintf("INSERT INTO %s (role_id, permission_alias) VALUES ('%s', '%s')  ON CONFLICT (role_id, permission_alias) DO NOTHING",
                self::RBAC_ROLE_PERMISSION_TABLE,
                $role,
                $alias)
        )->execute();
    }

    public function revokeAllPermissions(string $role)
    {
        $this->dbConnection->query(
            sprintf("DELETE FROM %s WHERE role_id = '%s'",
                self::RBAC_ROLE_PERMISSION_TABLE,
                $role)
        )->execute();
    }

    public function revokePermission(string $role, string $alias)
    {
        return $this->dbConnection->query(
            sprintf("DELETE FROM %s WHERE role_id = '%s' AND permission_alias = '%s'",
                self::RBAC_ROLE_PERMISSION_TABLE,
                $role,
                $alias));
    }

    public function isRolePermissionExist(string $role, string $alias)
    {
        $aliasExist = $this->dbConnection->query(
            sprintf("SELECT * FROM %s WHERE role_id = '%s' AND permission_alias = '%s'",
                self::RBAC_ROLE_PERMISSION_TABLE, $role, $alias))->rowCount();
        return $aliasExist > 0;
    }

    public function isPermissionExist(string $alias)
    {
        $aliasExist = $this->dbConnection->query(sprintf("SELECT * FROM %s WHERE alias = '%s'",
            self::RBAC_PERMISSIONS_TABLE, $alias))->rowCount();
        return $aliasExist > 0;
    }

    public function getPermissions()
    {
        return $this->dbConnection->query(sprintf('SELECT * FROM %s ORDER BY "application", "group", "name"',
            self::RBAC_PERMISSIONS_TABLE))->fetchAll(PDO::FETCH_ASSOC);
    }

}
