<?php
/**
 * @uri /oauth2/token
 */

class Oauth extends CfBaseResource
{

    function post($request)
    {
        $response = new Response($request);
        $server = Utils::getOauthServer();
        $oAuthResponse = $server->handleTokenRequest(OAuth2\Request::createFromGlobals());
        $response->body = $oAuthResponse->getResponseBody('json');
        $response->code = $oAuthResponse->getStatusCode();


        foreach ($oAuthResponse->getHttpHeaders() as $key => $value)
        {
            $response->addHeader($key, $value);
        }
        return $response;
    }

    function get($request)
    {
        $response = new Response($request);
        $this->accessRequired();

        $permissions = (new CfRBAC())->getPermissionsByRoles((new CfUsers($this->username))->getUserRoles());
        $getUserPermission = RbacAccessService::isActionAllowed('user.get', $permissions);

        $userData = Utils::get_rbac_settings($this->username, $getUserPermission);
        if (in_array('admin', $userData['roles']) === false)
        {
            throw new ResponseException('Permission denied ', Response::FORBIDDEN);
        }

        $server = Utils::getOauthServer();
        $accessTokenStorage = $server->getStorage('access_token');
        $tokens = $accessTokenStorage->getActiveTokens();
        // decode the json from PGSQL else we will encode it twice when encoding to output
        array_walk($tokens, function (&$value, $index)
        {
            $value['details'] = json_decode($value['details'], true);
        });
        $response->code = Response::OK;
        $response->body = json_encode($tokens);
        return $response;
    }
}

/**
 * @uri /oauth2/token/revoke
 */
class OauthRevoke extends CfBaseResource
{

    function post($request)
    {
        $this->accessRequired();
        $response = new Response($request);
        $data = Utils::getValidJsonData($request->data);
        $deleteToken = $data->tokens;

        $permissions = (new CfRBAC())->getPermissionsByRoles((new CfUsers($this->username))->getUserRoles());
        $getUserPermission = RbacAccessService::isActionAllowed('user.get', $permissions);

        $userData = Utils::get_rbac_settings($this->username, $getUserPermission);

        if (empty($deleteToken))
        {
            throw new Exception("Error Processing Request. tokens parameter is missing or empty", 1);
        }

        if (in_array('admin', $userData['roles']) === false)
        {
            // not in admin role so delete only if token is his own.
            if (count($deleteToken) !== 1)
            {
                throw new ResponseException('Permission denied to revoke multiple tokens.', Response::FORBIDDEN);
            }
            else
            {
                if (in_array($this->accessToken, $deleteToken) === false)
                {
                    throw new ResponseException('Permission denied to revoke token.', Response::FORBIDDEN);
                }
            }
        }
        syslog(LOG_DEBUG, 'DELETING ACCESS TOKEN:: ' . implode(',', $deleteToken));
        $server = Utils::getOauthServer();
        $accessTokenStorage = $server->getStorage('access_token');
        $accessTokenStorage->unsetAccessToken($deleteToken);
        $response->code = Response::OK;
        $response->body = null;
        return $response;
    }
}

