<?php

require_once "APIBaseTest.php";

class SettingsTest extends APIBaseTest
{

    public function testGetSettings()
    {
        try
        {
            $settings = $this->getResults('/settings');
            $this->assertValidJson($settings);
            $this->assertEquals(1, sizeof($settings));

            $this->assertEquals("debug", $settings[0]['logLevel']);
        }
        catch (Pest_Exception $e)
        {
            $this->fail($e);
        }
    }

    public function testUpdateSettingForBlueHostSetting()
    {
        try
        {
            $this->pest->post('/settings', '{
                "blueHostHorizon": 3600
                }');
            $this->assertEquals(204, $this->pest->lastStatus());

            $settings = $this->getResults('/settings');
            $this->assertValidJson($settings);
            $this->assertEquals(3600, $settings[0]['blueHostHorizon']);
        }
        catch (Pest_Exception $e)
        {
            $this->fail($e);
        }
    }

    public function testUpdateSettingRbac()
    {
        try
        {
            $this->pest->post('/settings', '{
                "rbacEnabled": false
                }');
            $this->assertEquals(204, $this->pest->lastStatus());

            $settings = $this->getResults('/settings');
            $this->assertValidJson($settings);
            $this->assertFalse($settings[0]['rbacEnabled']);
        }
        catch (Pest_Exception $e)
        {
            $this->fail($e);
        }
    }

    public function testUpdateSettingLogLevel()
    {
        try
        {
            $settings = $this->getResults('/settings');
            $this->assertValidJson($settings);
            $this->assertEquals(1, sizeof($settings));

            $this->assertEquals('debug', $settings[0]['logLevel']);

            $this->pest->post('/settings', '{
                "logLevel": "notice"
                }');
            $this->assertEquals(204, $this->pest->lastStatus());

            $settings = $this->getResults('/settings');
            $this->assertValidJson($settings);
            $this->assertEquals(1, sizeof($settings));

            $this->assertEquals('notice', $settings[0]['logLevel']);
        }
        catch (Pest_Exception $e)
        {
            $this->fail($e);
        }
    }

    public function testInsertSettingWithInvalidData()
    {
        try
        {
             $this->pest->put('/role/admin', '{
                     invalidjson
                }');

             $this->fail('Should trigger exception');

        }
        catch(Pest_Exception $e)
        {
            $this->assertEquals(400, $this->pest->lastStatus());
        }
    }

    public function testUpdateSettingWithInvalidData()
    {
        try
        {
             $this->pest->post('/role/admin', '{
                     invalidjson
                }');

             $this->fail('Should trigger exception');

        }
        catch(Pest_Exception $e)
        {
            $this->assertEquals(400, $this->pest->lastStatus());
        }
    }
}
