<?php


class health_diagnostic_model extends Cf_Model
{

    private $userName = false;

    public function __construct()
    {
        $this->load->driver('cache', array('adapter' => 'file'));
        parent::__construct();
    }


    public function setUserName($userName)
    {
        $this->userName = $userName;
    }

    public function getReportData($type)
    {
        $this->load->config('healthDiagnostic');
        $reportData = $this->config->item('healthDiagnosticReports')[$type];

        $result = json_decode(
            $this->getRestClient()->post('/health-diagnostic/report/' . $type, ['limit' => 50]),
            JSON_OBJECT_AS_ARRAY
        );

        return [
            'sql' => $result['data'][0]['query'],
            'title' => $reportData['title'],
            'description' => $reportData['description'],
            'recommendations' => $reportData['recommendations']
        ];

    }

    public function getHealthStatus()
    {
        $cacheKey = $this->userName . '_HealthStatus';
        if ($this->cache->get($cacheKey)) {
            return $this->cache->get($cacheKey);
        }
        $result = json_decode($this->getRestClient()->get('/health-diagnostic/status'), JSON_OBJECT_AS_ARRAY);
        $this->cache->save($cacheKey, $result, 300);
        return $result;
    }

    public function getHealthFailures($advancedReportsModel)
    {
        $cacheKey = $this->userName . '_HealthFailures';
        if ($this->cache->get($cacheKey)) {
            return $this->cache->get($cacheKey);
        }
        $data = $advancedReportsModel->runQuery($this->userName, 'SELECT hostkey, category FROM health_diagnostics_failures');
        $failedHosts = [];
        if (isset($data['rows']) && is_array($data)) {
            foreach ($data['rows'] as $row) {
                $failedHosts[$row[0]] = $row[1];
            }
        }

        $this->cache->save($cacheKey, $failedHosts, 120);
        return $failedHosts;
    }

    public function dismissHosts($hosts, $type)
    {
       $this->getRestClient()->post('/health-diagnostic/dismiss/' . $type, ['hosts' => $hosts]);
    }

    public function removeFromDismissed($hosts, $type)
    {
        $this->getRestClient()->delete('/health-diagnostic/dismiss/' . $type, ['hosts' => $hosts]);
    }

    public function getDismissedHosts($type)
    {
       return json_decode($this->getRestClient()->get('/health-diagnostic/dismiss/' . $type), JSON_OBJECT_AS_ARRAY);
    }
}
