<?php

namespace models;

use Adldap\Models\User;
use Illuminate\Contracts\Support\Arrayable;
use models\dataMappers\LdapUserAdapterInterface;

class LdapUserAdapter implements Arrayable, LdapUserAdapterInterface
{
    /**
     * @var $ldapUser User
     */
    private $ldapUser;
    private $config;

    public function __construct($config)
    {
        $this->config = $config;
    }

    public function setLdapUser($user){
        $this->ldapUser = $user;
    }

    public function getEmail()
    {
        return method_exists($this->ldapUser, 'getEmail') ? $this->ldapUser->getEmail() : '';
    }

    public function getId()
    {
        $loginAttribute = strtolower($this->config['login_attribute']);
        return $id = is_array($this->ldapUser->getAttribute($loginAttribute)) ? $this->ldapUser->getAttribute($loginAttribute)[0] : $this->ldapUser->getAttribute($loginAttribute);
    }

    public function getGroups()
    {
        $groups = [];
        if (!empty($this->config['group_attribute'])) {
            $memberOf = $this->ldapUser->getAttribute(strtolower($this->config['group_attribute']));
            if (!empty($memberOf)) {
                foreach ($memberOf as $group) {
                    // get common name from base DN
                    if (preg_match('/cn=(.+?),/i', $group, $matches)) {
                        $groups[] = $matches[1];
                    }
                }
            }
        }
        return $groups;
    }

    public function toArray()
    {
        return [
            'id' => $this->getId(),
            'email' => $this->getEmail(),
            'roles' => $this->getGroups()
        ];
    }
}