'use strict';

let modulesPageController = BaseBuildController.extend({
    init: function ($scope, modulesService, projectsService, $routeParams, $modal) {
        this._super($scope, projectsService, $routeParams, $modal);

        $scope.moduleName = $routeParams.name;
        this.$modal = $modal;
        this.projectsService = projectsService;
        this.modulesService = modulesService;
    },

    defineListeners: function () {
        this._super();
        let self = this;

        this.$scope.$watch('addedModules', function () {
            if (self.$scope.addedModules == undefined) return;
            self.$scope.addedVersion = null;
            self.$scope.addedModules.forEach(item => {
                if (item.name == self.$scope.moduleName && item.version) {
                    self.$scope.addedVersion = item.version;
                }
            });
        });

        this.$scope.$watch('addedVersion', function () {
            if (self.$scope.addedVersion === undefined) return;

            common.globalSpinner.show();
            self.modulesService.get(self.$scope.moduleName, self.$scope.addedVersion).then(
                response => {
                    self.$scope.module = response.data;
                    self.$scope.module.addedVersion = self.$scope.addedVersion;
                    Object.keys(self.$scope.module.versions).forEach(
                        version => {
                            if (self.$scope.module.versions[version].latest === true) {
                                self.$scope.module.latestVersion = version;
                            }
                        }
                    );

                    self.$scope.module.updated = (new Date(self.$scope.module.updated)).toLocaleString('default', {
                        month: 'short',
                        day: 'numeric',
                        year: 'numeric'
                    });
                    self.$scope.module.readme = htmlSpecialCharsDecode(self.$scope.module.readme);
                    common.globalSpinner.hide();
                },
                error => {
                    if (error.status == 404) {
                        $.notify('Module not found.');
                        common.globalSpinner.hide();
                    }
                }
            )
        });

       self.$scope.$on('module_data_saved', function () {
           self.refreshProjectStatus();
           self.loadProjectInfo(self.$scope.projectID);
        });

    },

    defineScope: function () {
        this.$scope.instance = "modulesPageController";

        this.$scope.openUpdateDialog = this.openUpdateDialog.bind(this);
        this.$scope.addModule = this.addModule.bind(this);
        this.$scope.removeModule = this.removeModule.bind(this);
    },

    openUpdateDialog: function () {
        let self = this;
        this.modalInstance = this.$modal.open({
            templateUrl: 'confirmModuleUpdate.html',
            backdrop: 'static',
            keyboard: true,
            windowClass: 'update-module-dialog',
            controller: function ($scope, $modalInstance, $parent) {
                $scope.module = $parent.$scope.module;
                $scope.commitMessage = `Updated ${$scope.module.name}\ from ${$scope.module.addedVersion}\ to ${$scope.module.latestVersion}`;
                $scope.close = function () {
                    $modalInstance.dismiss('cancel');
                };
                $scope.update = function () {
                    common.globalSpinner.show();
                    self.projectsService.updateModule(self.$scope.projectID, self.$scope.module.name, self.$scope.module.latestVersion)
                        .then(
                            () => {
                                $.notify('Module successfully updated.', {className: 'success'});
                                self.refreshProjectStatus();
                                self.loadProjectInfo(self.$scope.projectID);
                                self.$scope.addedVersion = self.$scope.module.latestVersion;
                                $modalInstance.dismiss('cancel');
                                common.globalSpinner.hide();
                            },
                            error => {
                                displayError(error, this.$scope);
                                common.globalSpinner.hide();
                                $modalInstance.dismiss('cancel');
                            }
                        )
                };
            },
            resolve: {
                $parent: function () {
                    return self;
                }
            }
        });
    },
    addModule: function () {
        common.globalSpinner.show();
        this.projectsService.addModule(this.$scope.projectID, this.$scope.module.name, this.$scope.module.version).then(
            () => {
                $.notify('Module successfully added.', {className: 'success'});
                this.refreshProjectStatus();
                this.loadProjectInfo(this.$scope.projectID);
                common.globalSpinner.hide();
            },
            error => {
                displayError(error, this.$scope);
                common.globalSpinner.hide();
            }
        )
    }
});

modulesPageController.$inject = ['$scope', 'modulesService', 'projectsService', '$routeParams', '$modal'];

let externalModulesPageController = BaseBuildController.extend({
    init: function ($scope, projectsService, $routeParams, $modal) {
        this._super($scope, projectsService, $routeParams, $modal);
        $scope.moduleName = atob($routeParams.name);
        this.$modal = $modal;
    },

    defineListeners: function () {
        this._super();
        let self = this;

        this.$scope.$watch('addedModules', function (addedModules) {
            if (addedModules == undefined) {
                return;
            }
            addedModules.forEach(module => {
                if (module.name == self.$scope.moduleName) {
                    self.$scope.module = module;
                    self.$scope.module.external = true;
                    self.$scope.module.addedVersion = self.$scope.module.version;
                }
            })
        }, true);
    },

    defineScope: function () {
        this.$scope.instance = "externalModulesPageController";

        this.$scope.removeModule = this.removeModule.bind(this);
    },

});

externalModulesPageController.$inject = ['$scope', 'projectsService', '$routeParams', '$modal'];
