angular.module('editableFieldModule', [])
    .directive('editableField', ['$rootScope', function ($rootScope) {
        return {
            restrict: 'A',
            scope: {
                model: '=',
                saveFunction: '&'
            },
            template: `<span class="editableField" ng-class="{clicked}">
                           <div ng-click="onClicked()" title="click to edit" ng-hide="clicked">{{ model }}</div>
                           <span class="editableField-form" ng-show="clicked"> 
                               <input title="" type="text" ng-model="model" />
                               <button ng-click="save()"><i class="bi bi-check"></i></button>
                               <button ng-click="cancel()"><i class="bi bi-x"></i></button>
                           </span>
                       </span>`,
            link: function (scope, element, attrs) {
                scope.clicked = false;
                let initialFieldValue = null;

                scope.onClicked = function () {
                    scope.clicked = true;
                    setTimeout(() => element[0].querySelector('input').select(), 100)
                }

                scope.save = function () {
                    scope.saveFunction().then(() => {
                        $.notify('Saved.', {className: "success"})
                        initialFieldValue = scope.model;
                    }, error => $.notify(error));
                    scope.clicked = false
                }

                scope.cancel = function () {
                    scope.model = initialFieldValue;
                    scope.clicked = false;
                }

                // set initial title value to revert changes in case of cancel action
                scope.$watch('model', function (field) {
                    if (field != undefined && initialFieldValue === null) {
                        initialFieldValue = field;
                    }
                })

                const eventCloseFn = (scope) => {
                    scope.cancel();
                    scope.$apply();
                }

                document.addEventListener('click', function (event) {
                    const outsideClick = !element[0].contains(event.target);
                    if (outsideClick) {
                        eventCloseFn(scope);
                    }
                });

                document.addEventListener('keyup', function (event) {
                    if (event.key === 'Escape') {
                        eventCloseFn(scope);
                    }
                });
            }
        };
    }]);
